#include "io.h"
#include "frame_buffer.h"

#define KEYBOARD_DATA_PORT 0x60

/** 
 * read_scan_code:
 * Reads a scan code from the keyboard.
 * 
 * @return The scan code (NOT an ASCII character!)
 */
u8int keyboard_read_scan_code(void) {
    return inb(KEYBOARD_DATA_PORT);
}

/** 
 * keyboard_scan_code_to_ascii:
 * Maps the scan code to an ASCII character for a standard US QWERTY keyboard.
 * 
 * @param scan_code The scan code to convert.
 * @return The corresponding ASCII character or 0 if unknown.
 */
u8int keyboard_scan_code_to_ascii(u8int scan_code) {
    // Ignore key releases (scan codes with bit 7 set)
    if (scan_code & 0x80) {
        return 0;
    }

    // Scan code to ASCII mapping for standard US QWERTY keyboard
    switch(scan_code) {
        // Numbers row
        case 0x02: return '1';
        case 0x03: return '2';
        case 0x04: return '3';
        case 0x05: return '4';
        case 0x06: return '5';
        case 0x07: return '6';
        case 0x08: return '7';
        case 0x09: return '8';
        case 0x0A: return '9';
        case 0x0B: return '0';
        case 0x0C: return '-';
        case 0x0D: return '=';
        case 0x0E: return '\b'; // Backspace
        
        // Top letter row
        case 0x10: return 'q';
        case 0x11: return 'w';
        case 0x12: return 'e';
        case 0x13: return 'r';
        case 0x14: return 't';
        case 0x15: return 'y';
        case 0x16: return 'u';
        case 0x17: return 'i';
        case 0x18: return 'o';
        case 0x19: return 'p';
        case 0x1A: return '[';
        case 0x1B: return ']';
        case 0x1C: return '\n'; // Enter
        
        // Middle letter row
        case 0x1E: return 'a';
        case 0x1F: return 's';
        case 0x20: return 'd';
        case 0x21: return 'f';
        case 0x22: return 'g';
        case 0x23: return 'h';
        case 0x24: return 'j';
        case 0x25: return 'k';
        case 0x26: return 'l';
        case 0x27: return ';';
        case 0x28: return '\'';
        case 0x29: return '`';
        
        // Bottom letter row
        case 0x2B: return '\\';
        case 0x2C: return 'z';
        case 0x2D: return 'x';
        case 0x2E: return 'c';
        case 0x2F: return 'v';
        case 0x30: return 'b';
        case 0x31: return 'n';
        case 0x32: return 'm';
        case 0x33: return ',';
        case 0x34: return '.';
        case 0x35: return '/';
        case 0x39: return ' '; // Spacebar
        
        // Numpad
        case 0x37: return '*'; // Numpad *
        case 0x47: return '7';
        case 0x48: return '8';
        case 0x49: return '9';
        case 0x4A: return '-';
        case 0x4B: return '4';
        case 0x4C: return '5';
        case 0x4D: return '6';
        case 0x4E: return '+';
        case 0x4F: return '1';
        case 0x50: return '2';
        case 0x51: return '3';
        case 0x52: return '0';
        case 0x53: return '.';
        
        default: return 0; // Unknown scan code
    }
}
