#include "io.h"  /* io.h is implemented in the section "Moving the cursor" */

/* The I/O ports */
#define FB_COMMAND_PORT        0x3D4    /* Command port for framebuffer */
#define FB_DATA_PORT           0x3D5    /* Data port for framebuffer */

/* The I/O port commands */
#define FB_HIGH_BYTE_COMMAND   0x0E    /* High byte command for framebuffer */
#define FB_LOW_BYTE_COMMAND    0x0F    /* Low byte command for framebuffer */

/**
 * fb_move_cursor:
 * Moves the cursor of the framebuffer to the given position.
 * This function uses I/O ports 0x3D4 and 0x3D5 to move the cursor.
 *
 * @param pos The new position of the cursor.
 */
void fb_move_cursor(unsigned short pos)
{
    /* Move the high byte of the position to the data port */
    outb(FB_COMMAND_PORT, FB_HIGH_BYTE_COMMAND); 
    outb(FB_DATA_PORT, ((pos >> 8) & 0x00FF));  /* Send the high byte */
    
    /* Move the low byte of the position to the data port */
    outb(FB_COMMAND_PORT, FB_LOW_BYTE_COMMAND);
    outb(FB_DATA_PORT, pos & 0x00FF);  /* Send the low byte */
}
