#include "io.h"

/* Define constants for screen dimensions */
#define SCREEN_WIDTH 80
#define SCREEN_HEIGHT 25
#define FB_BASE_ADDR 0x000B8000

/* Function to move the cursor (already provided in previous code) */
extern void fb_move_cursor(unsigned short pos);

/* Function to write a character to the framebuffer */
int write(char *buf, unsigned int len) {
    unsigned int i;
    static unsigned short cursor_pos = 0;  // Keep track of the cursor position
    
    for (i = 0; i < len; i++) {
        char c = buf[i];

        if (c == '\n') {
            /* Handle newline by moving the cursor to the next line */
            cursor_pos = (cursor_pos / SCREEN_WIDTH + 1) * SCREEN_WIDTH;
        } else {
            /* Write the character at the current cursor position */
            *((volatile unsigned short*) (FB_BASE_ADDR + 2 * cursor_pos)) = (unsigned short) c | 0x0F00;  // White foreground, black background
            cursor_pos++;
        }

        /* If the cursor is at the end of the screen, scroll */
        if (cursor_pos >= SCREEN_WIDTH * SCREEN_HEIGHT) {
            /* Scroll the screen up one row */
            for (unsigned int j = 0; j < (SCREEN_HEIGHT - 1) * SCREEN_WIDTH; j++) {
                *((volatile unsigned short*) (FB_BASE_ADDR + 2 * j)) = *((volatile unsigned short*) (FB_BASE_ADDR + 2 * (j + SCREEN_WIDTH)));
            }
            /* Clear the last row */
            for (unsigned int j = (SCREEN_HEIGHT - 1) * SCREEN_WIDTH; j < SCREEN_WIDTH * SCREEN_HEIGHT; j++) {
                *((volatile unsigned short*) (FB_BASE_ADDR + 2 * j)) = 0;
            }
            cursor_pos = (SCREEN_HEIGHT - 1) * SCREEN_WIDTH;  /* Move cursor to the last row */
        }

        /* Move the cursor to the new position */
        fb_move_cursor(cursor_pos);
    }

    return len;  /* Return the number of characters written */
}

/* Helper function to convert an integer to a string */
void itoa(int num, char *str, int base) {
    int i = 0;
    int isNegative = 0;

    // Handle 0 explicitly, otherwise empty string is printed
    if (num == 0) {
        str[i++] = '0';
        str[i] = '\0';
        return;
    }

    // Handle negative numbers only if base is 10
    if (num < 0 && base == 10) {
        isNegative = 1;
        num = -num;
    }

    // Process individual digits
    while (num != 0) {
        int rem = num % base;
        str[i++] = (rem > 9) ? (rem - 10) + 'a' : rem + '0';
        num = num / base;
    }

    // Append negative sign for negative numbers
    if (isNegative)
        str[i++] = '-';

    str[i] = '\0';  // Null-terminate string

    // Reverse the string
    int start = 0;
    int end = i - 1;
    while (start < end) {
        char temp = str[start];
        str[start] = str[end];
        str[end] = temp;
        start++;
        end--;
    }
}

/* The printf function */
int printf(const char *format, ...) {
    va_list args;
    va_start(args, format);
    char buf[1024];  // Buffer to hold the formatted output
    int buf_index = 0;
    int len = 0;

    while (*format != '\0') {
        if (*format == '%') {
            format++;  // Skip the '%' character

            if (*format == 'd') {  // Handle integers
                int num = va_arg(args, int);
                char str[32];
                itoa(num, str, 10);  // Convert integer to string
                for (int i = 0; str[i] != '\0'; i++) {
                    buf[buf_index++] = str[i];
                }
                len += buf_index;
            }
            else if (*format == 's') {  // Handle strings
                char *str = va_arg(args, char*);
                while (*str != '\0') {
                    buf[buf_index++] = *str++;
                }
                len += buf_index;
            }
            // Add other format specifiers like %x for hexadecimal, etc.
        } else {
            buf[buf_index++] = *format;  // Regular character, copy to buffer
        }
        format++;
    }

    // Write the buffered string to the framebuffer
    write(buf, buf_index);
    va_end(args);

    return len;
}
